#!/usr/bin/wish

# TCL/TK wrapper for photoslurp

##################################################################

# Build banner and copyright strings

set banner "PhotoSlurp Digital Photo Copier GUI"
set copyright "(C)2003-2016, Philip Munts, President, Munts AM Corp."

##################################################################

# Initialize some global vairables

set srcdir		""
set dstdir		""
set classifierlist	{DAY MONTH YEAR MODEL NONE}
set classifier		NONE
set separatelist	{NO YES}
set separatefiles	NO
set running		false

##################################################################

# Get environment variables

if {[info exist env(PHOTOSLURP_SRC)]} {
  set PHOTOSLURP_SRC $env(PHOTOSLURP_SRC)
  set env(PHOTOSLURP_SRC) ""
}

if {[info exist env(PHOTOSLURP_IMG)]} {
  set PHOTOSLURP_IMG $env(PHOTOSLURP_IMG)
  set env(PHOTOSLURP_IMG) ""
}

if {[info exist env(PHOTOSLURP_CLASSIFIER)]} {
  if {[lsearch -exact $classifierlist [string toupper $env(PHOTOSLURP_CLASSIFIER)]] > -1} {
    set PHOTOSLURP_CLASSIFIER [string toupper $env(PHOTOSLURP_CLASSIFIER)]
  }
  set env(PHOTOSLURP_CLASSIFIER) ""
}

if {[info exist env(PHOTOSLURP_SEPARATE)]} {
  if {[lsearch -exact $separatelist [string toupper $env(PHOTOSLURP_SEPARATE)]] > -1} {
    set PHOTOSLURP_SEPARATE [string toupper $env(PHOTOSLURP_SEPARATE)]
  }
  set env(PHOTOSLURP_SEPARATE) ""
}

##################################################################

# Setup main application window

wm title . PhotoSlurp

##################################################################

# Create operation buttons

frame .buttonframe -borderwidth 10
pack .buttonframe -side top -fill x -pady 5

button .buttonframe.slurp -text Slurp -command Slurp
pack .buttonframe.slurp -side left -padx 5

button .buttonframe.verify -text Verify -command Verify
pack .buttonframe.verify -side left -padx 5

button .buttonframe.dumpdb -text "Dump DB" -command DumpDB
pack .buttonframe.dumpdb -side left -padx 5

button .buttonframe.purgedb -text "Purge DB" -command PurgeDB
pack .buttonframe.purgedb -side left -padx 5

button .buttonframe.cleardb -text "Clear DB" -command ClearDB
pack .buttonframe.cleardb -side left -padx 5

button .buttonframe.about -text "About" -command About
pack .buttonframe.about -side left -padx 5

button .buttonframe.quit -text Quit -command exit
pack .buttonframe.quit -side left -padx 5

##################################################################

# Get source directory

#frame .srcdirframe
#pack .srcdirframe -side top -fill x -pady 5

#label .srcdirframe.srcdirlabel -text "Source Directory:"
#pack .srcdirframe.srcdirlabel -side left -padx 5

#entry .srcdirframe.srcdirbox -textvariable srcdir -state normal -width 100
#pack .srcdirframe.srcdirbox -side left -padx 5
#if {[info exists PHOTOSLURP_SRC]} {set srcdir $PHOTOSLURP_SRC}

#button .srcdirframe.srcdirbutton -text Choose -command ChooseSrcDir
#pack .srcdirframe.srcdirbutton -side left -padx 5

##################################################################

# Get destination directory

frame .dstdirframe
pack .dstdirframe -side top -fill x -pady 5

label .dstdirframe.dstdirlabel -text "Destination Directory:"
pack .dstdirframe.dstdirlabel -side left -padx 5

entry .dstdirframe.dstdirbox -textvariable dstdir -state normal -width 100
pack .dstdirframe.dstdirbox -side left -padx 5
if {[info exists PHOTOSLURP_IMG]} {set dstdir $PHOTOSLURP_IMG}

button .dstdirframe.dstdirbutton -text Choose -command ChooseDstDir
pack .dstdirframe.dstdirbutton -side left -padx 5

##################################################################

# Options frame

frame .optionsframe
pack .optionsframe -side top -fill x -pady 5

# Get classifier option

label .optionsframe.classifierlabel -text "Classifier:"
pack .optionsframe.classifierlabel -side left -padx 5

spinbox .optionsframe.classifierbox -values $classifierlist -textvariable classifier -state readonly -wrap true
pack .optionsframe.classifierbox -side left -padx 5
if {[info exists PHOTOSLURP_CLASSIFIER]} {
  .optionsframe.classifierbox set $PHOTOSLURP_CLASSIFIER
} else {
  .optionsframe.classifierbox set NONE
}

# Get separate files option

label .optionsframe.separatelabel -text "Separate files:"
pack .optionsframe.separatelabel -side left -padx 5

spinbox .optionsframe.separatebox -values $separatelist -textvariable separatefiles -state readonly -wrap true
pack .optionsframe.separatebox -side left -padx 5
if {[info exists PHOTOSLURP_SEPARATE]} {
  .optionsframe.separatebox set $PHOTOSLURP_SEPARATE
} else {
  .optionsframe.separatebox set NO
}

##################################################################

# Create text box for log output

frame .textframe
pack .textframe -side top -fill x -pady 5

set log [text .textframe.log -width 90 -height 20 -borderwidth 2 -font courier -relief raised -setgrid true -yscrollcommand {.textframe.scroll set}]
scrollbar .textframe.scroll -command {.textframe.log yview}
pack .textframe.scroll -side right -fill y
pack .textframe.log -side left -fill both -expand true
pack .textframe -side top -fill both -expand true

##################################################################

# Display banner stuff

$log delete 1.0 end
$log insert end $banner\n
$log insert end $copyright\n\n
$log see end

##################################################################

# Center the main window

tkwait visibility .

set x [expr [winfo screenwidth  .]/2 - [winfo width  .]/2]
set y [expr [winfo screenheight .]/2 - [winfo height .]/2 - 40]

wm geom . +$x+$y

# END OF MAINLINE CODE!

##################################################################

# Write to the log output text box

proc WriteLog {} {
  global input log running

  if [eof $input] {
    catch {close $input}
    set running false
  } else {
    gets $input line
    $log insert end $line\n
    $log see end
  }
}

##################################################################

# Run the C backend program and send its output to the log output text box

proc PhotoSlurp {command} {
  global running log banner copyright input srcdir dstdir classifier separatefiles

  if {$running} { return }

# Display banner stuff

  $log delete 1.0 end
  $log insert end $banner\n
  $log insert end $copyright\n\n
  $log see end

# Fixup Windows backslashes

  set srcdir [string map { \\ / } $srcdir]
  set dstdir [string map { \\ / } $dstdir]

# Build photoslurp command line

  set cmdline "|photoslurp"

  if {[string compare $srcdir ""]} { append cmdline " -s \"" $srcdir "\"" }
  if {[string compare $dstdir ""]} { append cmdline " -d \"" $dstdir "\"" }
  if {[string compare $classifier "NONE"]} { append cmdline " -c " $classifier }
  if {[string compare $separatefiles "NO"]} { append cmdline " -t" }

  append cmdline " " $command

# Run photoslurp

  if [catch {open $cmdline} input] {
    $log insert end $input\n
    $log see end
    set running false
  } else {
    set running true
    fileevent $input readable WriteLog
  }
}

##################################################################

# Trivial button handlers follow

proc Slurp {} {
  PhotoSlurp ""
}

proc Verify {} {
  PhotoSlurp "verify"
}

proc DumpDB {} {
  PhotoSlurp "dump"
}

proc PurgeDB {} {
  PhotoSlurp "purge"
}

proc ClearDB {} {
  PhotoSlurp "clear"
}

proc About {} {
  PhotoSlurp "info"
}

proc ChooseSrcDir {} {
  global running srcdir

  if {$running} { return }
  set srcdir [tk_chooseDirectory -title "Choose Source Directory" -initialdir $srcdir]
}

proc ChooseDstDir {} {
  global running dstdir

  if {$running} { return }
  set dstdir [tk_chooseDirectory -title "Choose Destination Directory" -initialdir $dstdir]
}
